/*
 *  <Short Description>
 *  Copyright (C) 2023  Brett Terpstra
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
#include <blt/gfx/model.h>
#include <blt/std/utility.h>

namespace blt::gfx
{
    static_dynamic_array::static_dynamic_array(static_dynamic_array&& move) noexcept
    {
        if (std::holds_alternative<GLuint[DATA_SIZE]>(move.data_)){
            data_ = std::move(std::get<GLuint[DATA_SIZE]>(move.data_));
        } else {
            data_ = std::get<GLuint*>(move.data_);
            size_ = move.size_;
        }
        move.data_ = nullptr;
        move.size_ = 0;
    }
    
    /*
     * -----------------------------------
     *      basic_vertex_array::vbo_t
     * -----------------------------------
     */
    basic_vertex_array::vbo_t::vbo_t()
    {
        glGenBuffers(1, &bufferID_);
    }
    
    basic_vertex_array::vbo_t::~vbo_t()
    {
        glDeleteBuffers(1, &bufferID_);
    }
    
    void basic_vertex_array::vbo_t::allocate(GLsizeiptr size, GLint buffer_type, GLint memory_type, void* data)
    {
        size_ = size;
        glBufferData(buffer_type, size, data, memory_type);
    }
    
    void blt::gfx::basic_vertex_array::vbo_t::update(GLsizeiptr size, void* data, GLint buffer_type, GLint memory_type)
    {
        if (size <= size_)
            sub_update(0, size, data, buffer_type);
        else
            allocate(size, buffer_type, memory_type, data);
        size_ = size;
    }
    
    void basic_vertex_array::vbo_t::sub_update(GLsizeiptr offset, GLsizeiptr size, void* data, GLint buffer_type)
    {
        glBufferSubData(buffer_type, offset, size, data);
    }
    
    void basic_vertex_array::vbo_t::bind(GLint buffer_type) const
    {
        glBindBuffer(buffer_type, bufferID_);
    }
    
    /*
     * ----------------------------
     *      basic_vertex_array
     * ----------------------------
     */
    
    blt::gfx::basic_vertex_array::basic_vertex_array()
    {
        glGenVertexArrays(1, &vaoID);
    }
}